@@ -15,6 +15,7 @@ class MediaGroupViewModel: MLBaseModel {
 
     var observable = Observable<MediaLibraryBaseModelObserver>()
 
+    var fileArrayLock = NSRecursiveLock()
     var files: [VLCMLMediaGroup]
 
     var cellType: BaseCollectionViewCell.Type {
@@ -28,26 +29,41 @@ class MediaGroupViewModel: MLBaseModel {
     var indicatorName: String = NSLocalizedString("VIDEO_GROUPS", comment: "")
 
     required init(medialibrary: MediaLibraryService) {
+        defer {
+            fileArrayLock.unlock()
+        }
         self.medialibrary = medialibrary
+        fileArrayLock.lock()
         files = medialibrary.medialib.mediaGroups() ?? []
         medialibrary.observable.addObserver(self)
     }
 
     func append(_ item: VLCMLMediaGroup) {
+        defer {
+            fileArrayLock.unlock()
+        }
+        fileArrayLock.lock()
         files.append(item)
     }
 
     func append(_ media: [VLCMLMedia], to mediaGroup: VLCMLMediaGroup) {
+        defer {
+            fileArrayLock.unlock()
+        }
         let originIds = originMediaGroupsIds(from: media)
 
         for medium in media {
             mediaGroup.add(medium)
         }
+        fileArrayLock.lock()
         files = swapModels(with: [mediaGroup])
         filterFilesFromDeletion(of: originIds)
     }
 
     func delete(_ items: [VLCMLMediaGroup]) {
+        defer {
+            fileArrayLock.unlock()
+        }
         for item in items {
             guard let media = item.media(of: .video) else {
                 continue
@@ -59,6 +75,7 @@ class MediaGroupViewModel: MLBaseModel {
             item.destroy()
         }
         medialibrary.reload()
+        fileArrayLock.lock()
         filterFilesFromDeletion(of: items)
     }
 
@@ -75,6 +92,10 @@ class MediaGroupViewModel: MLBaseModel {
     }
 
     func sort(by criteria: VLCMLSortingCriteria, desc: Bool) {
+        defer {
+            fileArrayLock.unlock()
+        }
+        fileArrayLock.lock()
         files = medialibrary.medialib.mediaGroups(with: criteria, desc: desc) ?? []
         sortModel.currentSort = criteria
         sortModel.desc = desc
@@ -89,7 +110,11 @@ class MediaGroupViewModel: MLBaseModel {
             assertionFailure("MediaGroupViewModel: Unable to create a mediagroup with name: \(name)")
             return false
         }
+        defer {
+            fileArrayLock.unlock()
+        }
         append(mediaGroup)
+        fileArrayLock.lock()
         content.forEach() { mediaGroup.add($0) }
         if let mediaGroupIds = mediaGroupIds {
             filterFilesFromDeletion(of: mediaGroupIds)
@@ -103,6 +128,9 @@ class MediaGroupViewModel: MLBaseModel {
     }
 
     func unGroupMedia(_ media: [VLCMLMedia], from originMediaGroup: VLCMLMediaGroup) -> Bool {
+        defer {
+            fileArrayLock.unlock()
+        }
         for medium in media {
             medium.removeFromGroup()
             guard let newGroup = medialibrary.medialib.createMediaGroup(withName: medium.title) else {
@@ -111,6 +139,7 @@ class MediaGroupViewModel: MLBaseModel {
             newGroup.add(medium)
         }
         if originMediaGroup.nbTotalMedia() == 0 {
+            fileArrayLock.lock()
             filterFilesFromDeletion(of: [originMediaGroup])
             medialibrary.medialib.deleteMediaGroup(withIdentifier: originMediaGroup.identifier())
         }
@@ -144,6 +173,10 @@ private extension MediaGroupViewModel {
 extension MediaGroupViewModel: MediaLibraryObserver {
     func medialibrary(_ medialibrary: MediaLibraryService,
                       didAddMediaGroups mediaGroups: [VLCMLMediaGroup]) {
+        defer {
+            fileArrayLock.unlock()
+        }
+        fileArrayLock.lock()
         for mediaGroup in mediaGroups {
             if !files.contains(where: { $0.identifier() == mediaGroup.identifier() }) {
                 files.append(mediaGroup)
@@ -156,6 +189,9 @@ extension MediaGroupViewModel: MediaLibraryObserver {
 
     func medialibrary(_ medialibrary: MediaLibraryService,
                       didModifyMediaGroupsWithIds mediaGroupsIds: [NSNumber]) {
+        defer {
+            fileArrayLock.unlock()
+        }
         var mediaGroups = [VLCMLMediaGroup]()
 
         mediaGroupsIds.forEach() {
@@ -166,6 +202,7 @@ extension MediaGroupViewModel: MediaLibraryObserver {
             mediaGroups.append(safeMediaGroup)
         }
 
+        fileArrayLock.lock()
         files = swapModels(with: mediaGroups)
         observable.observers.forEach() {
             $0.value.observer?.mediaLibraryBaseModelReloadView()
@@ -174,6 +211,10 @@ extension MediaGroupViewModel: MediaLibraryObserver {
 
     func medialibrary(_ medialibrary: MediaLibraryService,
                       didDeleteMediaGroupsWithIds mediaGroupsIds: [NSNumber]) {
+        defer {
+            fileArrayLock.unlock()
+        }
+        fileArrayLock.lock()
         files.removeAll {
             mediaGroupsIds.contains(NSNumber(value: $0.identifier()))
         }
