@@ -81,6 +81,14 @@ extension VLCURLHandler {
         downloadMovie(from: safeMovieURL, fileNameOfMedia: self.fileName)
     }
 
+    func cleanState() {
+        movieURL = nil
+        subURL = nil
+        successCallback = nil
+        errorCallback = nil
+        fileName = nil
+    }
+
     func parseURL(url: URL) {
         guard let query = url.query else {
             assertionFailure("VLCURLHandler: Invalid query.")
@@ -109,7 +117,7 @@ extension VLCURLHandler {
                  * while if the entire URL is encoded, it needs to be decoded before forwarding it to VLC */
                 let length = value.count
                 let end = value.index(value.startIndex, offsetBy: length > 14 ? 14 : length)
-                if (value.range(of: "%3A%2F%2F", options: NSString.CompareOptions.caseInsensitive, range: value.startIndex..<end, locale: nil) == nil) {
+                if value.range(of: "%3A%2F%2F", options: NSString.CompareOptions.caseInsensitive, range: value.startIndex..<end, locale: nil) == nil {
                     movieURL = URL(string: value)
                     break
                 }
@@ -331,6 +339,7 @@ class XCallbackURLHandler: NSObject, VLCURLHandler {
     @objc func performOpen(url: URL, options: [UIApplication.OpenURLOptionsKey: AnyObject]) -> Bool {
         let action = matchAction(action: url.path.replacingOccurrences(of: "/", with: ""))
 
+        cleanState()
         parseURL(url: url)
 
         switch action {
@@ -422,9 +431,8 @@ extension VLCURLHandler {
     // TODO: This code should probably not live here
     func play(url: URL, sub: URL? = nil, completion: ((Bool) -> Void)?) {
         let vpc = PlaybackService.sharedInstance()
-        if let mediaList = VLCMediaList(array: [VLCMedia(url: url)]) {
-            vpc.playMediaList(mediaList, firstIndex: 0, subtitlesFilePath: sub?.absoluteString, completion: completion)
-        }
+        let mediaList = VLCMediaList(array: [VLCMedia(url: url)])
+        vpc.playMediaList(mediaList, firstIndex: 0, subtitlesFilePath: sub?.absoluteString, completion: completion)
     }
 
 #if os(iOS)
